import java.util.List;

class ClassDesc extends ContentDesc {
    
    private static final byte FLAG_EXTERNALIZABLE = 0x04;
    private static final byte FLAG_SERIALIZABLE = 0x02;
    private static final byte FLAG_BLOCKDATA = 0x08;
    private static final byte FLAG_WRITE_METHOD = 0x01;
	
	private String className;
	private byte flags;
	private boolean externalizable;
	private boolean annotated;
	private boolean array;
	private ClassDesc superClassDesc;
	private List<FieldDesc> fields;
	
	public ClassDesc(String className, long serialVersionUID, byte flags, List<FieldDesc> fields) {
		this.className = className;
		array = className.charAt(0) == Unserialize.FT_ARRAY;
		this.flags = flags;
		externalizable = (flags & FLAG_EXTERNALIZABLE) != 0;
		boolean serializable = (flags & FLAG_SERIALIZABLE) != 0;
		if (externalizable == serializable) throw new IllegalArgumentException("bad flags (either both or neither externalizable nor serializable): + 0b" + Integer.toBinaryString(flags));
		annotated = externalizable ? ((flags & FLAG_BLOCKDATA) != 0) : ((flags & FLAG_WRITE_METHOD) != 0);
		if (externalizable ? ((flags & FLAG_WRITE_METHOD) != 0) : ((flags & FLAG_BLOCKDATA) != 0)) throw new IllegalArgumentException("bad flags (declared inappropriate annotation flag for type): + 0b" + Integer.toBinaryString(flags));
		this.fields = fields;
	}
	
	public boolean isExternalizable() {
		return externalizable;
	}
	
	public boolean isAnnotated() {
		return annotated;
	}
	
	public boolean isArray() {
		return array;
	}
	
	public String getName() {
		return className;
	}
	
	public byte getArrayElementFieldType() {
		if (!array) throw new IllegalStateException("not an array " + className);
		return (byte)className.charAt(1);
	}
	
	public String getArrayElementTypeName() {
		byte fieldType = getArrayElementFieldType();
		String typeName = decodeFieldType(fieldType);
		if (typeName == null) {
			typeName = className.substring(2, (className.length() - 1));
		}
		return typeName;
	}
	
	public static String decodeFieldType(byte fieldType) { // TODO: move this!
		switch (fieldType) {
			case Unserialize.FT_BYTE: return "byte";
			case Unserialize.FT_CHAR: return "char";
			case Unserialize.FT_DOUBLE: return "double";
			case Unserialize.FT_FLOAT: return "float";
			case Unserialize.FT_INT: return "int";
			case Unserialize.FT_LONG: return "long";
			case Unserialize.FT_SHORT: return "short";
			case Unserialize.FT_BOOLEAN: return "boolean";
			case Unserialize.FT_ARRAY: return null;
			case Unserialize.FT_OBJECT: return null;
			default: throw new IllegalStateException("bad field type: 0x" + Integer.toHexString(fieldType & 0xff) + " (" + (char)fieldType + ")");
		}
	}
	
	public ClassDesc getSuperClass() {
		return superClassDesc;
	}
	
	public void setSuperClass(ClassDesc superClassDesc) {
		this.superClassDesc = superClassDesc;
	}
	
	public int getDepth() {
		if (superClassDesc == null) return 1;
		else return superClassDesc.getDepth() + 1;
	}
	
	public List<FieldDesc> getFields() {
		return fields;
	}
	
	public int hashCode() {
		return className.hashCode();
	}
	
	public boolean equals(Object obj) {
		if (obj == this) return true;
		if (!(obj instanceof ClassDesc)) return false;
		ClassDesc that = (ClassDesc)obj;
		return this.className.equals(that.className);
	}
	
}
