import java.io.OutputStreamWriter;
import java.io.IOException;
import java.util.List;
import java.util.Map;

public class ContentWriter {
	
	private IndentingBufferedWriter out;
	
	public ContentWriter(IndentingBufferedWriter out) {
		this.out = out;
	}
	
	public void write(List<ContentDesc> contents, List<ContentDesc> objects) throws IOException {
		dumpContents(contents);
		out.newLine();
		int handle = Unserialize.FIRST_HANDLE;
		for (ContentDesc obj: objects) {
			out.write(Unserialize.formatHandle(handle));
			out.write(": ");
			if (obj instanceof ClassDesc) {
				ClassDesc classDesc = (ClassDesc)obj;
				out.write("class ");
				out.write(classDesc.getName());
			}
			else if (obj instanceof ObjectDesc) {
				ObjectDesc objectDesc = (ObjectDesc)obj;
				out.write("instance of ");
				out.write(objectDesc.getClassDesc().getName());
				out.write(" {");
				out.newLine();
				out.indent();
				for (ClassData cdata: objectDesc.getClassData()) {
					for (Map.Entry<FieldDesc, Object> field: cdata.getValues().entrySet()) {
						FieldDesc fieldDesc = field.getKey();
						out.write(fieldDesc.getTypeName());
						out.write(" ");
						out.write(fieldDesc.getName());
						out.write(" = ");
						dumpValue(field.getValue(), ';');
						out.newLine();
					}
					List<ContentDesc> annotation = cdata.getAnnotation();
					if (annotation != null) {
						out.write(cdata.getClassDesc().getName());
						out.write(" ");
						dumpContents(annotation);
					}
				}
				out.dedent();
				out.write("}");
			}
			else if (obj instanceof ArrayDesc) {
				ArrayDesc arrayDesc = (ArrayDesc)obj;
				out.write("array of ");
				out.write(arrayDesc.getElementType());
				out.write(" [");
				out.newLine();
				out.indent();
				for (Object element: arrayDesc.getElements()) {
					dumpValue(element, ',');
					out.newLine();
				}
				out.dedent();
				out.write("]");
			}
			else if (obj instanceof StringDesc) {
				StringDesc stringDesc = (StringDesc)obj;
				out.write('"');
				out.write(stringDesc.getString());
				out.write('"');
			}
			else if (obj instanceof BlockDataDesc) {
				out.write("block data");
			}
			else {
				throw new IOException("unknown kind of content (" + obj.getClass() + "): " + obj);
			}
			out.newLine();
			out.newLine();
			++handle;
		}
		out.flush();
	}
	
	private void dumpContents(List<ContentDesc> contents) throws IOException {
		out.write("[");
		out.newLine();
		out.indent();
		for (ContentDesc obj: contents) {
			if (obj instanceof BlockDataDesc) {
				BlockDataDesc blockDataDesc = (BlockDataDesc)obj;
				out.write('<');
				byte[] data = blockDataDesc.getData();
				for (int i = 0; i < data.length; ++i) {
					if (i > 0) out.write(' ');
					byte b = data[i];
					if (b < 0x10) out.write('0');
					out.write(Integer.toHexString(b));
				}
				out.write(">,");
			}
			else {
				dumpValue(obj, ',');
			}
			out.newLine();
		}
		out.dedent();
		out.write("]");
		out.newLine();
	}
	
	private void dumpValue(Object obj, char terminator) throws IOException {
		if (obj instanceof ContentDesc) {
			out.write(Unserialize.formatHandle((ContentDesc)obj));
			out.write(terminator);
			out.write(" // ");
			if (obj instanceof ObjectDesc) {
				ObjectDesc objectDesc = (ObjectDesc)obj;
				out.write("instance of ");
				out.write(objectDesc.getClassDesc().getName());
			}
			else if (obj instanceof ArrayDesc) {
				ArrayDesc arrayDesc = (ArrayDesc)obj;
				out.write("array of ");
				out.write(arrayDesc.getElementType());
			}
			else if (obj instanceof StringDesc) {
				StringDesc stringDesc = (StringDesc)obj;
				out.write('"');
				out.write(stringDesc.getString());
				out.write('"');
			}
			else {
				throw new IOException("unknown kind of content (" + obj.getClass() + "): " + obj);
			}
		}
		else {
			out.write(String.valueOf(obj));
			out.write(terminator);
		}
	}
	
}
