class FieldDesc {
	
	public static boolean isObject(byte fieldType) {
		switch (fieldType) {
			case Unserialize.FT_BYTE:
			case Unserialize.FT_CHAR:
			case Unserialize.FT_DOUBLE:
			case Unserialize.FT_FLOAT:
			case Unserialize.FT_INT:
			case Unserialize.FT_LONG:
			case Unserialize.FT_SHORT:
			case Unserialize.FT_BOOLEAN:
				return false;
			case Unserialize.FT_ARRAY:
			case Unserialize.FT_OBJECT:
				return true;
			default:
				throw new IllegalArgumentException("bad field type: 0x" + Integer.toHexString(fieldType & 0xff) + " (" + (char)fieldType + ")");
		}
	}
	
	private byte typeCode;
	private String name;
	private StringDesc fieldClass;
	
	public FieldDesc(byte typeCode, String name, StringDesc fieldClass) {
		this.typeCode = typeCode;
		this.name = name;
		this.fieldClass = fieldClass;
	}
	
	public byte getType() {
		return typeCode;
	}
	
	public String getTypeName() {
		String typeName;
		if (isObject(typeCode)) {
			typeName = fieldClass.getString();
		}
		else {
			typeName = new String(new char[] {(char)typeCode});
		}
		int arrayDepth = 0;
		while (typeName.charAt(arrayDepth) == Unserialize.FT_ARRAY) {
			++arrayDepth;
		}
		if (arrayDepth > 0) typeName = typeName.substring(arrayDepth);
		if (typeName.startsWith("L")) {
			assert typeName.endsWith(";");
			typeName = typeName.substring(1, typeName.length() - 1).replace('/', '.');
		}
		else {
			assert typeName.length() == 1;
			typeName = ClassDesc.decodeFieldType((byte)typeName.charAt(0));
			assert typeName != null;
		}
		for (int i = 0; i < arrayDepth; ++i) typeName = typeName + "[]";
		return typeName;
	}
	
	public String getName() {
		return name;
	}
	
	public int hashCode() {
		return name.hashCode();
	}
	
	public boolean equals(Object obj) {
		if (obj == this) return true;
		if (!(obj instanceof FieldDesc)) return false;
		FieldDesc that = (FieldDesc)obj;
		return this.name.equals(that.name);
	}
	
}
