import java.io.BufferedWriter;
import java.io.Writer;
import java.io.IOException;

public class IndentingBufferedWriter extends BufferedWriter {
	
	private int indent;
	private boolean atLineStart;
	
	public IndentingBufferedWriter(Writer out, int sz) {
		super(out, sz);
		atLineStart = true;
	}
	
	public IndentingBufferedWriter(Writer out) {
		super(out);
		atLineStart = true;
	}
	
	public void indent() {
		++indent;
	}
	
	public void dedent() {
		if (indent == 0) throw new IllegalStateException();
		--indent;
	}
	
	public void newLine() throws IOException {
		super.newLine();
		atLineStart = true;
	}
	
	public void write(char[] cbuf, int off, int len) throws IOException {
		writeIndentation();
		super.write(cbuf, off, len);
	}
	
	public void write(int c) throws IOException {
		writeIndentation();
		super.write(c);
	}
	
	public void write(String s, int off, int len) throws IOException {
		writeIndentation();
		super.write(s, off, len);
	}
	
	private void writeIndentation() throws IOException {
		if (atLineStart) {
			for (int i = 0; i < indent; ++i) super.write('\t');
			atLineStart = false;
		}
	}
	
}
